/*--- DOCCITT2.C  ------------------- Listing 4 ---* 
 * Compute CCITT-CRC on-the-fly.         
 * Usage:   doccitt2 filename
 *
 * Author: Nigel Cort (with help from Andrew Chalk, PhD)
 * (c) 1990 C Gazette. Use freely if authorship acknowledged
 *--------------------------------------------------*/
 
#include <stdio.h>

unsigned short do_crc ( unsigned short, unsigned short );

void main ( int argc, char * argv[] )
{
   FILE *fin;            /* file were reading into buff         */                
   char *buff;           /* buffer we're working on             */                    
   size_t i, j;          /* counters of bytes in buff           */        
   unsigned short crc;   /* the CRC value being computed        */               

   if ( argc < 2 )
   {
      fputs ( "Error! Must specify filename.", stderr );
      exit ( 5 );
   }

   if (( fin = fopen (argv[1], "rb")) == NULL )
   {
      fprintf ( stderr, "Cannot open %s", argv[1] );
      exit ( 6 );
   }

   /*--------------------------------------------------
    * Set up a very large input buffer of 32K bytes.
    * This program does no good if it doesn't fly!
    --------------------------------------------------*/
   if (( buff = (char *) malloc (32766)) == NULL )
   {
      fputs ( " Out of memory ", stderr );
      exit ( 7 );
   }

   crc = 0;

   for (;;)
   {
      i = fread ( buff, 1, 32766, fin );
      if ( i == 0 )
      {
         if ( feof ( fin ))  /* we're done, so show results */
         {
            printf ( "CCITT CRC for %s is %04X", argv[1], crc );
            exit ( 0 );
         }
         else         /* read another 32K of file */
            continue;
      }
            
      for ( j = 0 ; j < i; j ++ )  /* loop through the buffer */
            crc = do_crc ( crc, buff [j] ); /* Unlike table version */
   }
}

/*-----------------------------------------------
 *  Compute the CCITT CRC on-the-fly               
 *  Note absence of any table.
 *----------------------------------------------*/
unsigned short do_crc ( unsigned short crc, unsigned short ch )
{
    static unsigned int i;
    ch <<= 8;                               /* Move to MSB       */
    for ( i = 8; i > 0; i-- )               /* Go through 8 bits */
    {
        if ( (ch ^ crc) & 0X8000 )          /* Perform CRC calc. */
             crc = ( crc << 1 ) ^ 0x1021;   
        else 
             crc <<= 1;
        ch <<= 1;
    }
    return ( crc );
}

